@@ -27,13 +27,13 @@
 #include "mozilla/Attributes.h"
 
 class nsDataDocumentContentPolicy final : public nsIContentPolicy {
-  ~nsDataDocumentContentPolicy() {}
+  ~nsDataDocumentContentPolicy() = default;
 
  public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSICONTENTPOLICY
 
-  nsDataDocumentContentPolicy() {}
+  nsDataDocumentContentPolicy() = default;
 };
 
 #endif /* nsDataDocumentContentPolicy_h__ */
