@@ -22,10 +22,12 @@
 #  include "nsDirectoryServiceDefs.h"
 #endif
 
+#include "nsAppRunner.h"
+#include "mozilla/ipc/ProcessUtils.h"
+
 using mozilla::ipc::IOThreadChild;
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 #if defined(XP_WIN) && defined(MOZ_SANDBOX)
 static void SetTmpEnvironmentVariable(nsIFile* aValue) {
@@ -84,6 +86,8 @@ bool ContentProcess::Init(int aArgc, char* aArgv[]) {
   char* prefMapHandle = nullptr;
   char* prefsLen = nullptr;
   char* prefMapSize = nullptr;
+  char* jsInitHandle = nullptr;
+  char* jsInitLen = nullptr;
 #if defined(XP_MACOSX) && defined(MOZ_SANDBOX)
   nsCOMPtr<nsIFile> profileDir;
 #endif
@@ -139,6 +143,19 @@ bool ContentProcess::Init(int aArgc, char* aArgv[]) {
         return false;
       }
       prefMapSize = aArgv[i];
+
+    } else if (strcmp(aArgv[i], "-jsInit") == 0) {
+      // command line: -jsInit [handle] length
+#ifdef XP_WIN
+      if (++i == aArgc) {
+        return false;
+      }
+      jsInitHandle = aArgv[i];
+#endif
+      if (++i == aArgc) {
+        return false;
+      }
+      jsInitLen = aArgv[i];
     } else if (strcmp(aArgv[i], "-safeMode") == 0) {
       gSafeMode = true;
 
@@ -169,14 +186,18 @@ bool ContentProcess::Init(int aArgc, char* aArgv[]) {
     return false;
   }
 
-  SharedPreferenceDeserializer deserializer;
+  ::mozilla::ipc::SharedPreferenceDeserializer deserializer;
   if (!deserializer.DeserializeFromSharedMemory(prefsHandle, prefMapHandle,
                                                 prefsLen, prefMapSize)) {
     return false;
   }
 
-  mContent.Init(IOThreadChild::message_loop(), ParentPid(), *parentBuildID,
-                IOThreadChild::TakeChannel(), *childID, *isForBrowser);
+  if (!::mozilla::ipc::ImportSharedJSInit(jsInitHandle, jsInitLen)) {
+    return false;
+  }
+
+  mContent.Init(ParentPid(), *parentBuildID, IOThreadChild::TakeInitialPort(),
+                *childID, *isForBrowser);
 
   mXREEmbed.Start();
 #if (defined(XP_MACOSX)) && defined(MOZ_SANDBOX)
@@ -199,5 +220,4 @@ bool ContentProcess::Init(int aArgc, char* aArgv[]) {
 // in ContentChild::ActorDestroy().
 void ContentProcess::CleanUp() { mXREEmbed.Stop(); }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
