@@ -5,21 +5,25 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "WMFMediaDataDecoder.h"
+
 #include "VideoUtils.h"
 #include "WMFUtils.h"
+#include "mozilla/Logging.h"
+#include "mozilla/ProfilerMarkers.h"
+#include "mozilla/SyncRunnable.h"
+#include "mozilla/TaskQueue.h"
 #include "mozilla/Telemetry.h"
 #include "nsTArray.h"
 
-#include "mozilla/Logging.h"
-#include "mozilla/SyncRunnable.h"
-
 #define LOG(...) MOZ_LOG(sPDMLog, mozilla::LogLevel::Debug, (__VA_ARGS__))
 
 namespace mozilla {
 
-WMFMediaDataDecoder::WMFMediaDataDecoder(MFTManager* aMFTManager,
-                                         TaskQueue* aTaskQueue)
-    : mTaskQueue(aTaskQueue), mMFTManager(aMFTManager) {}
+WMFMediaDataDecoder::WMFMediaDataDecoder(MFTManager* aMFTManager)
+    : mTaskQueue(
+          new TaskQueue(GetMediaThreadPool(MediaThreadType::PLATFORM_DECODER),
+                        "WMFMediaDataDecoder")),
+      mMFTManager(aMFTManager) {}
 
 WMFMediaDataDecoder::~WMFMediaDataDecoder() {}
 
@@ -51,36 +55,22 @@ static void SendTelemetry(unsigned long hr) {
   } else {
     sample = 3;  // high bucket
   }
-
-  nsCOMPtr<nsIRunnable> runnable =
-      NS_NewRunnableFunction("SendTelemetry", [sample] {
-        Telemetry::Accumulate(Telemetry::MEDIA_WMF_DECODE_ERROR, sample);
-      });
-
-  SystemGroup::Dispatch(TaskCategory::Other, runnable.forget());
 }
 
 RefPtr<ShutdownPromise> WMFMediaDataDecoder::Shutdown() {
   MOZ_DIAGNOSTIC_ASSERT(!mIsShutDown);
-
   mIsShutDown = true;
 
-  if (mTaskQueue) {
-    return InvokeAsync(mTaskQueue, this, __func__,
-                       &WMFMediaDataDecoder::ProcessShutdown);
-  }
-  return ProcessShutdown();
-}
-
-RefPtr<ShutdownPromise> WMFMediaDataDecoder::ProcessShutdown() {
-  if (mMFTManager) {
-    mMFTManager->Shutdown();
-    mMFTManager = nullptr;
-    if (!mRecordedError && mHasSuccessfulOutput) {
-      SendTelemetry(S_OK);
+  return InvokeAsync(mTaskQueue, __func__, [self = RefPtr{this}, this] {
+    if (mMFTManager) {
+      mMFTManager->Shutdown();
+      mMFTManager = nullptr;
+      if (!mRecordedError && mHasSuccessfulOutput) {
+        SendTelemetry(S_OK);
+      }
     }
-  }
-  return ShutdownPromise::CreateAndResolve(true, __func__);
+    return mTaskQueue->BeginShutdown();
+  });
 }
 
 // Inserts data into the decoder's pipeline.
@@ -94,11 +84,19 @@ RefPtr<MediaDataDecoder::DecodePromise> WMFMediaDataDecoder::Decode(
 
 RefPtr<MediaDataDecoder::DecodePromise> WMFMediaDataDecoder::ProcessError(
     HRESULT aError, const char* aReason) {
+  MOZ_ASSERT(mTaskQueue->IsCurrentThreadIn());
   if (!mRecordedError) {
     SendTelemetry(aError);
     mRecordedError = true;
   }
 
+  nsPrintfCString markerString(
+      "WMFMediaDataDecoder::ProcessError for decoder with description %s with "
+      "reason: %s",
+      GetDescriptionName().get(), aReason);
+  LOG(markerString.get());
+  PROFILER_MARKER_TEXT("WMFDecoder Error", MEDIA_PLAYBACK, {}, markerString);
+
   // TODO: For the error DXGI_ERROR_DEVICE_RESET, we could return
   // NS_ERROR_DOM_MEDIA_NEED_NEW_DECODER to get the latest device. Maybe retry
   // up to 3 times.
@@ -110,6 +108,7 @@ RefPtr<MediaDataDecoder::DecodePromise> WMFMediaDataDecoder::ProcessError(
 
 RefPtr<MediaDataDecoder::DecodePromise> WMFMediaDataDecoder::ProcessDecode(
     MediaRawData* aSample) {
+  MOZ_ASSERT(mTaskQueue->IsCurrentThreadIn());
   DecodedData results;
   HRESULT hr = mMFTManager->Input(aSample);
   if (hr == MF_E_NOTACCEPTING) {
@@ -125,10 +124,24 @@ RefPtr<MediaDataDecoder::DecodePromise> WMFMediaDataDecoder::ProcessDecode(
     return ProcessError(hr, "MFTManager::Input");
   }
 
+  // MFT decoder sometime will return incorrect first output to us, which always
+  // has 0 timestamp, even if the input we gave to MFT has timestamp that is way
+  // later than 0. In order to distinguish that incorrect output with the real
+  // first frame, we set the seek threshold to the first sample time that will
+  // help us ignore any output time that is earlier than the first input time.
+  // Only does that when manager doesn't have seek threshold, because we don't
+  // want to mess up the seeking triggered from the higher level of media stack.
+  if (!mHasGuardedAgainstIncorrectFirstSample &&
+      !mMFTManager->HasSeekThreshold()) {
+    mHasGuardedAgainstIncorrectFirstSample = true;
+    mMFTManager->SetSeekThreshold(aSample->mTime);
+  }
+
   if (!mLastTime || aSample->mTime > *mLastTime) {
     mLastTime = Some(aSample->mTime);
     mLastDuration = aSample->mDuration;
   }
+
   mSamplesCount++;
   mDrainStatus = DrainStatus::DRAINABLE;
   mLastStreamOffset = aSample->mOffset;
@@ -142,6 +155,7 @@ RefPtr<MediaDataDecoder::DecodePromise> WMFMediaDataDecoder::ProcessDecode(
 
 HRESULT
 WMFMediaDataDecoder::ProcessOutput(DecodedData& aResults) {
+  MOZ_ASSERT(mTaskQueue->IsCurrentThreadIn());
   RefPtr<MediaData> output;
   HRESULT hr = S_OK;
   while (SUCCEEDED(hr = mMFTManager->Output(mLastStreamOffset, output))) {
@@ -156,12 +170,14 @@ WMFMediaDataDecoder::ProcessOutput(DecodedData& aResults) {
 }
 
 RefPtr<MediaDataDecoder::FlushPromise> WMFMediaDataDecoder::ProcessFlush() {
+  MOZ_ASSERT(mTaskQueue->IsCurrentThreadIn());
   if (mMFTManager) {
     mMFTManager->Flush();
   }
   mDrainStatus = DrainStatus::DRAINED;
   mSamplesCount = 0;
   mLastTime.reset();
+  mHasGuardedAgainstIncorrectFirstSample = false;
   return FlushPromise::CreateAndResolve(true, __func__);
 }
 
@@ -173,6 +189,7 @@ RefPtr<MediaDataDecoder::FlushPromise> WMFMediaDataDecoder::Flush() {
 }
 
 RefPtr<MediaDataDecoder::DecodePromise> WMFMediaDataDecoder::ProcessDrain() {
+  MOZ_ASSERT(mTaskQueue->IsCurrentThreadIn());
   if (!mMFTManager || mDrainStatus == DrainStatus::DRAINED) {
     return DecodePromise::CreateAndResolve(DecodedData(), __func__);
   }
@@ -208,6 +225,20 @@ RefPtr<MediaDataDecoder::DecodePromise> WMFMediaDataDecoder::ProcessDrain() {
         // set the duration of the last sample as it was input.
         data->mDuration = mLastDuration;
       }
+    } else if (results.Length() == 1 &&
+               results.LastElement()->mType == MediaData::Type::AUDIO_DATA) {
+      // When we drain the audio decoder and one frame was queued (such as with
+      // AAC) the MFT will re-calculate the starting time rather than use the
+      // value set on the IMF Sample.
+      // This is normally an okay thing to do; however when dealing with poorly
+      // muxed content that has incorrect start time, it could lead to broken
+      // A/V sync. So we ensure that we use the compressed sample's time
+      // instead. Additionally, this is what all other audio decoders are doing
+      // anyway.
+      MOZ_ASSERT(mLastTime,
+                 "We must have attempted to decode at least one frame to get "
+                 "one decoded output");
+      results.LastElement()->As<AudioData>()->SetOriginalStartTime(*mLastTime);
     }
     return DecodePromise::CreateAndResolve(std::move(results), __func__);
   }
@@ -234,6 +265,7 @@ void WMFMediaDataDecoder::SetSeekThreshold(const media::TimeUnit& aTime) {
   RefPtr<WMFMediaDataDecoder> self = this;
   nsCOMPtr<nsIRunnable> runnable = NS_NewRunnableFunction(
       "WMFMediaDataDecoder::SetSeekThreshold", [self, aTime]() {
+        MOZ_ASSERT(self->mTaskQueue->IsCurrentThreadIn());
         media::TimeUnit threshold = aTime;
         self->mMFTManager->SetSeekThreshold(threshold);
       });
