@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -11,20 +11,17 @@
 #include "nsIFileURL.h"
 #include "nsIStandardURL.h"
 #include "mozilla/Encoding.h"
-#include "nsIObserver.h"
 #include "nsCOMPtr.h"
 #include "nsURLHelper.h"
-#include "nsIClassInfo.h"
 #include "nsISizeOf.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/LinkedList.h"
 #include "mozilla/MemoryReporting.h"
-#include "nsIIPCSerializableURI.h"
 #include "nsISensitiveInfoHiddenURI.h"
 #include "nsIURIMutator.h"
 
 #ifdef NS_BUILD_REFCNT_LOGGING
-#define DEBUG_DUMP_URLS_AT_SHUTDOWN
+#  define DEBUG_DUMP_URLS_AT_SHUTDOWN
 #endif
 
 class nsIBinaryInputStream;
@@ -45,9 +42,7 @@ namespace net {
 class nsStandardURL : public nsIFileURL,
                       public nsIStandardURL,
                       public nsISerializable,
-                      public nsIClassInfo,
                       public nsISizeOf,
-                      public nsIIPCSerializableURI,
                       public nsISensitiveInfoHiddenURI
 #ifdef DEBUG_DUMP_URLS_AT_SHUTDOWN
     ,
@@ -56,25 +51,21 @@ class nsStandardURL : public nsIFileURL,
 {
  protected:
   virtual ~nsStandardURL();
+  explicit nsStandardURL(bool aSupportsFileURL = false, bool aTrackURL = true);
 
  public:
-  NS_DECL_ISUPPORTS
+  NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIURI
   NS_DECL_NSIURL
   NS_DECL_NSIFILEURL
   NS_DECL_NSISTANDARDURL
   NS_DECL_NSISERIALIZABLE
-  NS_DECL_NSICLASSINFO
-  NS_DECL_NSIMUTABLE
-  NS_DECL_NSIIPCSERIALIZABLEURI
   NS_DECL_NSISENSITIVEINFOHIDDENURI
 
   // nsISizeOf
   virtual size_t SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const override;
   virtual size_t SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const override;
 
-  explicit nsStandardURL(bool aSupportsFileURL = false, bool aTrackURL = true);
-
   static void InitGlobalObjects();
   static void ShutdownGlobalObjects();
 
@@ -83,12 +74,12 @@ class nsStandardURL : public nsIFileURL,
   // location and length of an url segment relative to mSpec
   //
   struct URLSegment {
-    uint32_t mPos;
-    int32_t mLen;
+    uint32_t mPos{0};
+    int32_t mLen{-1};
 
-    URLSegment() : mPos(0), mLen(-1) {}
+    URLSegment() = default;
     URLSegment(uint32_t pos, int32_t len) : mPos(pos), mLen(len) {}
-    URLSegment(const URLSegment &aCopy) : mPos(aCopy.mPos), mLen(aCopy.mLen) {}
+    URLSegment(const URLSegment& aCopy) = default;
     void Reset() {
       mPos = 0;
       mLen = -1;
@@ -96,8 +87,8 @@ class nsStandardURL : public nsIFileURL,
     // Merge another segment following this one to it if they're contiguous
     // Assumes we have something like "foo;bar" where this object is 'foo' and
     // right is 'bar'.
-    void Merge(const nsCString &spec, const char separator,
-               const URLSegment &right) {
+    void Merge(const nsCString& spec, const char separator,
+               const URLSegment& right) {
       if (mLen >= 0 && *(spec.get() + mPos + mLen) == separator &&
           mPos + mLen + 1 == right.mPos) {
         mLen += 1 + right.mLen;
@@ -105,46 +96,32 @@ class nsStandardURL : public nsIFileURL,
     }
   };
 
-  //
-  // Pref observer
-  //
-  class nsPrefObserver final : public nsIObserver {
-    ~nsPrefObserver() {}
-
-   public:
-    NS_DECL_ISUPPORTS
-    NS_DECL_NSIOBSERVER
-
-    nsPrefObserver() {}
-  };
-  friend class nsPrefObserver;
-
   //
   // URL segment encoder : performs charset conversion and URL escaping.
   //
   class nsSegmentEncoder {
    public:
-    explicit nsSegmentEncoder(const Encoding *encoding = nullptr);
+    explicit nsSegmentEncoder(const Encoding* encoding = nullptr);
 
     // Encode the given segment if necessary, and return the length of
-    // the encoded segment.  The encoded segment is appended to |buf|
+    // the encoded segment.  The encoded segment is appended to |aOut|
     // if and only if encoding is required.
-    int32_t EncodeSegmentCount(const char *str, const URLSegment &segment,
-                               int16_t mask, nsCString &buf, bool &appended,
+    int32_t EncodeSegmentCount(const char* str, const URLSegment& aSeg,
+                               int16_t mask, nsCString& aOut, bool& appended,
                                uint32_t extraLen = 0);
 
     // Encode the given string if necessary, and return a reference to
-    // the encoded string.  Returns a reference to |buf| if encoding
+    // the encoded string.  Returns a reference to |result| if encoding
     // is required.  Otherwise, a reference to |str| is returned.
-    const nsACString &EncodeSegment(const nsACString &str, int16_t mask,
-                                    nsCString &buf);
+    const nsACString& EncodeSegment(const nsACString& str, int16_t mask,
+                                    nsCString& result);
 
    private:
-    const Encoding *mEncoding;
+    const Encoding* mEncoding;
   };
   friend class nsSegmentEncoder;
 
-  static nsresult NormalizeIPv4(const nsACString &host, nsCString &result);
+  static nsresult NormalizeIPv4(const nsACString& host, nsCString& result);
 
  protected:
   // enum used in a few places to specify how .ref attribute should be handled
@@ -152,18 +129,18 @@ class nsStandardURL : public nsIFileURL,
 
   // Helper to share code between Equals and EqualsExceptRef
   // NOTE: *not* virtual, because no one needs to override this so far...
-  nsresult EqualsInternal(nsIURI *unknownOther, RefHandlingEnum refHandlingMode,
-                          bool *result);
+  nsresult EqualsInternal(nsIURI* unknownOther, RefHandlingEnum refHandlingMode,
+                          bool* result);
 
-  virtual nsStandardURL *StartClone();
+  virtual nsStandardURL* StartClone();
 
   // Helper to share code between Clone methods.
   nsresult CloneInternal(RefHandlingEnum aRefHandlingMode,
-                         const nsACString &newRef, nsIURI **aClone);
+                         const nsACString& aNewRef, nsIURI** aClone);
   // Helper method that copies member variables from the source StandardURL
   // if copyCached = true, it will also copy mFile and mDisplayHost
-  nsresult CopyMembers(nsStandardURL *source, RefHandlingEnum mode,
-                       const nsACString &newRef, bool copyCached = false);
+  nsresult CopyMembers(nsStandardURL* source, RefHandlingEnum mode,
+                       const nsACString& newRef, bool copyCached = false);
 
   // Helper for subclass implementation of GetFile().  Subclasses that map
   // URIs to files in a special way should implement this method.  It should
@@ -171,31 +148,33 @@ class nsStandardURL : public nsIFileURL,
   // returns NS_ERROR_NO_INTERFACE if the url does not map to a file
   virtual nsresult EnsureFile();
 
-  virtual nsresult SetSpecInternal(const nsACString &input);
-  virtual nsresult SetScheme(const nsACString &input);
-  virtual nsresult SetUserPass(const nsACString &input);
-  virtual nsresult SetUsername(const nsACString &input);
-  virtual nsresult SetPassword(const nsACString &input);
-  virtual nsresult SetHostPort(const nsACString &aValue);
-  virtual nsresult SetHost(const nsACString &input);
+  virtual nsresult Clone(nsIURI** aURI);
+  virtual nsresult SetSpecInternal(const nsACString& input);
+  virtual nsresult SetScheme(const nsACString& input);
+  virtual nsresult SetUserPass(const nsACString& input);
+  virtual nsresult SetUsername(const nsACString& input);
+  virtual nsresult SetPassword(const nsACString& input);
+  virtual nsresult SetHostPort(const nsACString& aValue);
+  virtual nsresult SetHost(const nsACString& input);
   virtual nsresult SetPort(int32_t port);
-  virtual nsresult SetPathQueryRef(const nsACString &input);
-  virtual nsresult SetRef(const nsACString &input);
-  virtual nsresult SetFilePath(const nsACString &input);
-  virtual nsresult SetQuery(const nsACString &input);
-  virtual nsresult SetQueryWithEncoding(const nsACString &input,
-                                        const Encoding *encoding);
-  bool Deserialize(const mozilla::ipc::URIParams &);
+  virtual nsresult SetPathQueryRef(const nsACString& input);
+  virtual nsresult SetRef(const nsACString& input);
+  virtual nsresult SetFilePath(const nsACString& input);
+  virtual nsresult SetQuery(const nsACString& input);
+  virtual nsresult SetQueryWithEncoding(const nsACString& input,
+                                        const Encoding* encoding);
+  bool Deserialize(const mozilla::ipc::URIParams&);
+  nsresult ReadPrivate(nsIObjectInputStream* stream);
 
  private:
-  nsresult Init(uint32_t urlType, int32_t defaultPort, const nsACString &spec,
-                const char *charset, nsIURI *baseURI);
+  nsresult Init(uint32_t urlType, int32_t defaultPort, const nsACString& spec,
+                const char* charset, nsIURI* baseURI);
   nsresult SetDefaultPort(int32_t aNewDefaultPort);
-  nsresult SetFile(nsIFile *file);
+  nsresult SetFile(nsIFile* file);
 
-  nsresult SetFileNameInternal(const nsACString &input);
-  nsresult SetFileBaseNameInternal(const nsACString &input);
-  nsresult SetFileExtensionInternal(const nsACString &input);
+  nsresult SetFileNameInternal(const nsACString& input);
+  nsresult SetFileBaseNameInternal(const nsACString& input);
+  nsresult SetFileExtensionInternal(const nsACString& input);
 
   int32_t Port() { return mPort == -1 ? mDefaultPort : mPort; }
 
@@ -203,60 +182,60 @@ class nsStandardURL : public nsIFileURL,
   void Clear();
   void InvalidateCache(bool invalidateCachedFile = true);
 
-  bool ValidIPv6orHostname(const char *host, uint32_t aLen);
+  bool ValidIPv6orHostname(const char* host, uint32_t length);
   static bool IsValidOfBase(unsigned char c, const uint32_t base);
-  nsresult NormalizeIDN(const nsACString &host, nsCString &result);
+  nsresult NormalizeIDN(const nsACString& host, nsCString& result);
   nsresult CheckIfHostIsAscii();
-  void CoalescePath(netCoalesceFlags coalesceFlag, char *path);
+  void CoalescePath(netCoalesceFlags coalesceFlag, char* path);
 
-  uint32_t AppendSegmentToBuf(char *, uint32_t, const char *,
-                              const URLSegment &input, URLSegment &output,
-                              const nsCString *esc = nullptr,
-                              bool useEsc = false, int32_t *diff = nullptr);
-  uint32_t AppendToBuf(char *, uint32_t, const char *, uint32_t);
+  uint32_t AppendSegmentToBuf(char*, uint32_t, const char*,
+                              const URLSegment& input, URLSegment& output,
+                              const nsCString* esc = nullptr,
+                              bool useEsc = false, int32_t* diff = nullptr);
+  uint32_t AppendToBuf(char*, uint32_t, const char*, uint32_t);
 
-  nsresult BuildNormalizedSpec(const char *spec, const Encoding *encoding);
-  nsresult SetSpecWithEncoding(const nsACString &input,
-                               const Encoding *encoding);
+  nsresult BuildNormalizedSpec(const char* spec, const Encoding* encoding);
+  nsresult SetSpecWithEncoding(const nsACString& input,
+                               const Encoding* encoding);
 
-  bool SegmentIs(const URLSegment &s1, const char *val,
+  bool SegmentIs(const URLSegment& seg, const char* val,
                  bool ignoreCase = false);
-  bool SegmentIs(const char *spec, const URLSegment &s1, const char *val,
-                 bool ignoreCase = false);
-  bool SegmentIs(const URLSegment &s1, const char *val, const URLSegment &s2,
+  bool SegmentIs(const char* spec, const URLSegment& seg, const char* val,
                  bool ignoreCase = false);
+  bool SegmentIs(const URLSegment& seg1, const char* val,
+                 const URLSegment& seg2, bool ignoreCase = false);
 
-  int32_t ReplaceSegment(uint32_t pos, uint32_t len, const char *val,
+  int32_t ReplaceSegment(uint32_t pos, uint32_t len, const char* val,
                          uint32_t valLen);
-  int32_t ReplaceSegment(uint32_t pos, uint32_t len, const nsACString &val);
+  int32_t ReplaceSegment(uint32_t pos, uint32_t len, const nsACString& val);
 
-  nsresult ParseURL(const char *spec, int32_t specLen);
-  nsresult ParsePath(const char *spec, uint32_t pathPos, int32_t pathLen = -1);
+  nsresult ParseURL(const char* spec, int32_t specLen);
+  nsresult ParsePath(const char* spec, uint32_t pathPos, int32_t pathLen = -1);
 
-  char *AppendToSubstring(uint32_t pos, int32_t len, const char *tail);
+  char* AppendToSubstring(uint32_t pos, int32_t len, const char* tail);
 
   // dependent substring helpers
-  const nsDependentCSubstring Segment(uint32_t pos, int32_t len);  // see below
-  const nsDependentCSubstring Segment(const URLSegment &s) {
+  nsDependentCSubstring Segment(uint32_t pos, int32_t len);  // see below
+  nsDependentCSubstring Segment(const URLSegment& s) {
     return Segment(s.mPos, s.mLen);
   }
 
   // dependent substring getters
-  const nsDependentCSubstring Prepath();  // see below
-  const nsDependentCSubstring Scheme() { return Segment(mScheme); }
-  const nsDependentCSubstring Userpass(bool includeDelim = false);  // see below
-  const nsDependentCSubstring Username() { return Segment(mUsername); }
-  const nsDependentCSubstring Password() { return Segment(mPassword); }
-  const nsDependentCSubstring Hostport();  // see below
-  const nsDependentCSubstring Host();      // see below
-  const nsDependentCSubstring Path() { return Segment(mPath); }
-  const nsDependentCSubstring Filepath() { return Segment(mFilepath); }
-  const nsDependentCSubstring Directory() { return Segment(mDirectory); }
-  const nsDependentCSubstring Filename();  // see below
-  const nsDependentCSubstring Basename() { return Segment(mBasename); }
-  const nsDependentCSubstring Extension() { return Segment(mExtension); }
-  const nsDependentCSubstring Query() { return Segment(mQuery); }
-  const nsDependentCSubstring Ref() { return Segment(mRef); }
+  nsDependentCSubstring Prepath();  // see below
+  nsDependentCSubstring Scheme() { return Segment(mScheme); }
+  nsDependentCSubstring Userpass(bool includeDelim = false);  // see below
+  nsDependentCSubstring Username() { return Segment(mUsername); }
+  nsDependentCSubstring Password() { return Segment(mPassword); }
+  nsDependentCSubstring Hostport();  // see below
+  nsDependentCSubstring Host();      // see below
+  nsDependentCSubstring Path() { return Segment(mPath); }
+  nsDependentCSubstring Filepath() { return Segment(mFilepath); }
+  nsDependentCSubstring Directory() { return Segment(mDirectory); }
+  nsDependentCSubstring Filename();  // see below
+  nsDependentCSubstring Basename() { return Segment(mBasename); }
+  nsDependentCSubstring Extension() { return Segment(mExtension); }
+  nsDependentCSubstring Query() { return Segment(mQuery); }
+  nsDependentCSubstring Ref() { return Segment(mRef); }
 
   // shift the URLSegments to the right by diff
   void ShiftFromAuthority(int32_t diff);
@@ -272,18 +251,22 @@ class nsStandardURL : public nsIFileURL,
   void ShiftFromRef(int32_t diff);
 
   // fastload helper functions
-  nsresult ReadSegment(nsIBinaryInputStream *, URLSegment &);
-  nsresult WriteSegment(nsIBinaryOutputStream *, const URLSegment &);
+  nsresult ReadSegment(nsIBinaryInputStream*, URLSegment&);
+  nsresult WriteSegment(nsIBinaryOutputStream*, const URLSegment&);
 
-  static void PrefsChanged(nsIPrefBranch *prefs, const char *pref);
+  void FindHostLimit(nsACString::const_iterator& aStart,
+                     nsACString::const_iterator& aEnd);
 
-  void FindHostLimit(nsACString::const_iterator &aStart,
-                     nsACString::const_iterator &aEnd);
+  // Asserts that the URL has sane values
+  void SanityCheck();
+
+  // Checks if the URL has a valid representation.
+  bool IsValid();
 
   // mSpec contains the normalized version of the URL spec (UTF-8 encoded).
   nsCString mSpec;
-  int32_t mDefaultPort;
-  int32_t mPort;
+  int32_t mDefaultPort{-1};
+  int32_t mPort{-1};
 
   // url parts (relative to mSpec)
   URLSegment mScheme;
@@ -311,20 +294,15 @@ class nsStandardURL : public nsIFileURL,
 
   enum { eEncoding_Unknown, eEncoding_ASCII, eEncoding_UTF8 };
 
-  uint32_t mSpecEncoding : 2;     // eEncoding_xxx
   uint32_t mURLType : 2;          // nsIStandardURL::URLTYPE_xxx
-  uint32_t mMutable : 1;          // nsIStandardURL::mutable
   uint32_t mSupportsFileURL : 1;  // QI to nsIFileURL?
   uint32_t mCheckedIfHostA : 1;   // If set to true, it means either that
                                   // mDisplayHost has a been initialized, or
                                   // that the hostname is not punycode
 
-  // global objects.  don't use COMPtr as its destructor will cause a
-  // coredump if we leak it.
-  static nsIIDNService *gIDN;
+  // global objects.
+  static StaticRefPtr<nsIIDNService> gIDN;
   static const char gHostLimitDigits[];
-  static bool gInitialized;
-  static bool gPunycodeHost;
 
  public:
 #ifdef DEBUG_DUMP_URLS_AT_SHUTDOWN
@@ -339,25 +317,32 @@ class nsStandardURL : public nsIFileURL,
                            public BaseURIMutator<T>,
                            public nsIStandardURLMutator,
                            public nsIURLMutator,
-                           public nsIFileURLMutator {
+                           public nsIFileURLMutator,
+                           public nsISerializable {
     NS_FORWARD_SAFE_NSIURISETTERS_RET(BaseURIMutator<T>::mURI)
 
-    MOZ_MUST_USE NS_IMETHOD
-    Deserialize(const mozilla::ipc::URIParams &aParams) override {
+    [[nodiscard]] NS_IMETHOD Deserialize(
+        const mozilla::ipc::URIParams& aParams) override {
       return BaseURIMutator<T>::InitFromIPCParams(aParams);
     }
 
-    MOZ_MUST_USE NS_IMETHOD Read(nsIObjectInputStream *aStream) override {
+    NS_IMETHOD
+    Write(nsIObjectOutputStream* aOutputStream) override {
+      MOZ_ASSERT_UNREACHABLE("Use nsIURIMutator.read() instead");
+      return NS_ERROR_NOT_IMPLEMENTED;
+    }
+
+    [[nodiscard]] NS_IMETHOD Read(nsIObjectInputStream* aStream) override {
       return BaseURIMutator<T>::InitFromInputStream(aStream);
     }
 
-    MOZ_MUST_USE NS_IMETHOD Finalize(nsIURI **aURI) override {
+    [[nodiscard]] NS_IMETHOD Finalize(nsIURI** aURI) override {
       BaseURIMutator<T>::mURI.forget(aURI);
       return NS_OK;
     }
 
-    MOZ_MUST_USE NS_IMETHOD SetSpec(const nsACString &aSpec,
-                                    nsIURIMutator **aMutator) override {
+    [[nodiscard]] NS_IMETHOD SetSpec(const nsACString& aSpec,
+                                     nsIURIMutator** aMutator) override {
       if (aMutator) {
         nsCOMPtr<nsIURIMutator> mutator = this;
         mutator.forget(aMutator);
@@ -365,10 +350,10 @@ class nsStandardURL : public nsIFileURL,
       return BaseURIMutator<T>::InitFromSpec(aSpec);
     }
 
-    MOZ_MUST_USE NS_IMETHOD Init(uint32_t aURLType, int32_t aDefaultPort,
-                                 const nsACString &aSpec, const char *aCharset,
-                                 nsIURI *aBaseURI,
-                                 nsIURIMutator **aMutator) override {
+    [[nodiscard]] NS_IMETHOD Init(uint32_t aURLType, int32_t aDefaultPort,
+                                  const nsACString& aSpec, const char* aCharset,
+                                  nsIURI* aBaseURI,
+                                  nsIURIMutator** aMutator) override {
       if (aMutator) {
         nsCOMPtr<nsIURIMutator> mutator = this;
         mutator.forget(aMutator);
@@ -378,19 +363,19 @@ class nsStandardURL : public nsIFileURL,
         // We don't need a new URI object if we already have one
         BaseURIMutator<T>::mURI.swap(uri);
       } else {
-        uri = new T();
+        uri = Create();
       }
       nsresult rv =
           uri->Init(aURLType, aDefaultPort, aSpec, aCharset, aBaseURI);
       if (NS_FAILED(rv)) {
         return rv;
       }
-      BaseURIMutator<T>::mURI = uri.forget();
+      BaseURIMutator<T>::mURI = std::move(uri);
       return NS_OK;
     }
 
-    MOZ_MUST_USE NS_IMETHODIMP
-    SetDefaultPort(int32_t aNewDefaultPort, nsIURIMutator **aMutator) override {
+    [[nodiscard]] NS_IMETHODIMP SetDefaultPort(
+        int32_t aNewDefaultPort, nsIURIMutator** aMutator) override {
       if (!BaseURIMutator<T>::mURI) {
         return NS_ERROR_NULL_POINTER;
       }
@@ -401,8 +386,8 @@ class nsStandardURL : public nsIFileURL,
       return BaseURIMutator<T>::mURI->SetDefaultPort(aNewDefaultPort);
     }
 
-    MOZ_MUST_USE NS_IMETHOD SetFileName(const nsACString &aFileName,
-                                        nsIURIMutator **aMutator) override {
+    [[nodiscard]] NS_IMETHOD SetFileName(const nsACString& aFileName,
+                                         nsIURIMutator** aMutator) override {
       if (!BaseURIMutator<T>::mURI) {
         return NS_ERROR_NULL_POINTER;
       }
@@ -413,8 +398,8 @@ class nsStandardURL : public nsIFileURL,
       return BaseURIMutator<T>::mURI->SetFileNameInternal(aFileName);
     }
 
-    MOZ_MUST_USE NS_IMETHOD SetFileBaseName(const nsACString &aFileBaseName,
-                                            nsIURIMutator **aMutator) override {
+    [[nodiscard]] NS_IMETHOD SetFileBaseName(
+        const nsACString& aFileBaseName, nsIURIMutator** aMutator) override {
       if (!BaseURIMutator<T>::mURI) {
         return NS_ERROR_NULL_POINTER;
       }
@@ -425,8 +410,8 @@ class nsStandardURL : public nsIFileURL,
       return BaseURIMutator<T>::mURI->SetFileBaseNameInternal(aFileBaseName);
     }
 
-    MOZ_MUST_USE NS_IMETHOD SetFileExtension(
-        const nsACString &aFileExtension, nsIURIMutator **aMutator) override {
+    [[nodiscard]] NS_IMETHOD SetFileExtension(
+        const nsACString& aFileExtension, nsIURIMutator** aMutator) override {
       if (!BaseURIMutator<T>::mURI) {
         return NS_ERROR_NULL_POINTER;
       }
@@ -437,13 +422,20 @@ class nsStandardURL : public nsIFileURL,
       return BaseURIMutator<T>::mURI->SetFileExtensionInternal(aFileExtension);
     }
 
-    MOZ_MUST_USE NS_IMETHOD SetFile(nsIFile *aFile) override {
+    T* Create() override { return new T(mMarkedFileURL); }
+
+    [[nodiscard]] NS_IMETHOD MarkFileURL() override {
+      mMarkedFileURL = true;
+      return NS_OK;
+    }
+
+    [[nodiscard]] NS_IMETHOD SetFile(nsIFile* aFile) override {
       RefPtr<T> uri;
       if (BaseURIMutator<T>::mURI) {
         // We don't need a new URI object if we already have one
         BaseURIMutator<T>::mURI.swap(uri);
       } else {
-        uri = new T(/* supportsFileURL = */ true);
+        uri = new T(/* aSupportsFileURL = */ true);
       }
 
       nsresult rv = uri->SetFile(aFile);
@@ -454,10 +446,12 @@ class nsStandardURL : public nsIFileURL,
       return NS_OK;
     }
 
-    explicit TemplatedMutator() {}
+    explicit TemplatedMutator() = default;
 
    private:
-    virtual ~TemplatedMutator() {}
+    virtual ~TemplatedMutator() = default;
+
+    bool mMarkedFileURL = false;
 
     friend T;
   };
@@ -485,8 +479,7 @@ class nsStandardURL : public nsIFileURL,
 // Dependent substring getters
 //-----------------------------------------------------------------------------
 
-inline const nsDependentCSubstring nsStandardURL::Segment(uint32_t pos,
-                                                          int32_t len) {
+inline nsDependentCSubstring nsStandardURL::Segment(uint32_t pos, int32_t len) {
   if (len < 0) {
     pos = 0;
     len = 0;
@@ -494,25 +487,32 @@ inline const nsDependentCSubstring nsStandardURL::Segment(uint32_t pos,
   return Substring(mSpec, pos, uint32_t(len));
 }
 
-inline const nsDependentCSubstring nsStandardURL::Prepath() {
+inline nsDependentCSubstring nsStandardURL::Prepath() {
   uint32_t len = 0;
   if (mAuthority.mLen >= 0) len = mAuthority.mPos + mAuthority.mLen;
   return Substring(mSpec, 0, len);
 }
 
-inline const nsDependentCSubstring nsStandardURL::Userpass(bool includeDelim) {
+inline nsDependentCSubstring nsStandardURL::Userpass(bool includeDelim) {
   uint32_t pos = 0, len = 0;
-  // if there is no username, then there can be no password
-  if (mUsername.mLen > 0) {
-    pos = mUsername.mPos;
-    len = mUsername.mLen;
-    if (mPassword.mLen >= 0) len += (mPassword.mLen + 1);
+  if (mUsername.mLen > 0 || mPassword.mLen > 0) {
+    if (mUsername.mLen > 0) {
+      pos = mUsername.mPos;
+      len = mUsername.mLen;
+      if (mPassword.mLen >= 0) {
+        len += (mPassword.mLen + 1);
+      }
+    } else {
+      pos = mPassword.mPos - 1;
+      len = mPassword.mLen + 1;
+    }
+
     if (includeDelim) len++;
   }
   return Substring(mSpec, pos, len);
 }
 
-inline const nsDependentCSubstring nsStandardURL::Hostport() {
+inline nsDependentCSubstring nsStandardURL::Hostport() {
   uint32_t pos = 0, len = 0;
   if (mAuthority.mLen > 0) {
     pos = mHost.mPos;
@@ -521,7 +521,7 @@ inline const nsDependentCSubstring nsStandardURL::Hostport() {
   return Substring(mSpec, pos, len);
 }
 
-inline const nsDependentCSubstring nsStandardURL::Host() {
+inline nsDependentCSubstring nsStandardURL::Host() {
   uint32_t pos = 0, len = 0;
   if (mHost.mLen > 0) {
     pos = mHost.mPos;
@@ -534,7 +534,7 @@ inline const nsDependentCSubstring nsStandardURL::Host() {
   return Substring(mSpec, pos, len);
 }
 
-inline const nsDependentCSubstring nsStandardURL::Filename() {
+inline nsDependentCSubstring nsStandardURL::Filename() {
   uint32_t pos = 0, len = 0;
   // if there is no basename, then there can be no extension
   if (mBasename.mLen > 0) {
