@@ -9,20 +9,23 @@
 #include "mozilla/ipc/FileDescriptor.h"
 #include "mozilla/net/SocketProcessParent.h"
 #include "mozilla/psm/PSMIPCTypes.h"
+#include "mozilla/Preferences.h"
 #include "mozilla/Unused.h"
 #include "nsICaptivePortalService.h"
 #include "nsIParentalControlsService.h"
 #include "nsINetworkLinkService.h"
 #include "nsIObserverService.h"
 #include "nsIOService.h"
+#include "nsNetCID.h"
 #include "TRRService.h"
 
 namespace mozilla {
 namespace net {
 
 static const char* gTRRUriCallbackPrefs[] = {
-    "network.trr.uri", "network.trr.mode", kRolloutURIPref, kRolloutURIPref,
-    nullptr,
+    "network.trr.uri",  "network.trr.default_provider_uri",
+    "network.trr.mode", kRolloutURIPref,
+    kRolloutModePref,   nullptr,
 };
 
 NS_IMPL_ISUPPORTS(TRRServiceParent, nsIObserver, nsISupportsWeakReference)
@@ -41,7 +44,10 @@ void TRRServiceParent::Init() {
     return;
   }
 
-  TRRService::AddObserver(this);
+  nsCOMPtr<nsIObserverService> obs =
+      static_cast<nsIObserverService*>(gIOService);
+  TRRService::AddObserver(this, obs);
+
   bool captiveIsPassed = TRRService::CheckCaptivePortalIsPassed();
   bool parentalControlEnabled = TRRService::GetParentalControlEnabledInternal();
 
@@ -63,69 +69,18 @@ void TRRServiceParent::Init() {
 NS_IMETHODIMP
 TRRServiceParent::Observe(nsISupports* aSubject, const char* aTopic,
                           const char16_t* aData) {
-  if (!strcmp(aTopic, kOpenCaptivePortalLoginEvent)) {
-    Unused << SendNotifyObserver(
-        nsDependentCString(aTopic),
-        aData ? nsDependentString(aData) : VoidString());
-  } else if (!strcmp(aTopic, NS_CAPTIVE_PORTAL_CONNECTIVITY)) {
-    if (!mTRRBLStorageInited) {
-      RefPtr<DataStorage> storage =
-          DataStorage::Get(DataStorageClass::TRRBlacklist);
-      if (storage) {
-        nsresult rv = storage->Init(nullptr);
-        if (NS_WARN_IF(NS_FAILED(rv))) {
-          Unused << SendNotifyObserver(
-              nsDependentCString(aTopic),
-              aData ? nsDependentString(aData) : VoidString());
-          return NS_OK;
-        }
-
-        psm::DataStorageEntry entry;
-        storage->GetAll(&entry.items());
-
-        RefPtr<TRRServiceParent> self = this;
-        nsCString topic(aTopic);
-        nsString data(aData);
-        rv = storage->AsyncTakeFileDesc(
-            [self, entry, topic, data](ipc::FileDescriptor&& aWriteFd) {
-              // We need to send this message before
-              // NS_CAPTIVE_PORTAL_CONNECTIVITY notification to make sure
-              // TRRBLStorage got inited properly.
-              Unused << self->SendInitTRRBLStorage(entry, aWriteFd);
-              Unused << self->SendNotifyObserver(topic, data);
-              self->mTRRBLStorageInited = true;
-            });
-
-        if (NS_SUCCEEDED(rv)) {
-          return NS_OK;
-        }
-      }
-    }
-
-    Unused << SendNotifyObserver(
-        nsDependentCString(aTopic),
-        aData ? nsDependentString(aData) : VoidString());
-  } else if (!strcmp(aTopic, kClearPrivateData) || !strcmp(aTopic, kPurge)) {
-    Unused << SendNotifyObserver(
-        nsDependentCString(aTopic),
-        aData ? nsDependentString(aData) : VoidString());
-  } else if (!strcmp(aTopic, NS_DNS_SUFFIX_LIST_UPDATED_TOPIC) ||
-             !strcmp(aTopic, NS_NETWORK_LINK_TOPIC)) {
+  if (!strcmp(aTopic, NS_DNS_SUFFIX_LIST_UPDATED_TOPIC) ||
+      !strcmp(aTopic, NS_NETWORK_LINK_TOPIC)) {
     nsCOMPtr<nsINetworkLinkService> link = do_QueryInterface(aSubject);
     // The network link service notification normally passes itself as the
     // subject, but some unit tests will sometimes pass a null subject.
     if (link) {
       nsTArray<nsCString> suffixList;
       link->GetDnsSuffixList(suffixList);
-      bool platformDisabledTRR = TRRService::CheckPlatformDNSStatus(link);
-      Unused << SendUpdatePlatformDNSInformation(suffixList,
-                                                 platformDisabledTRR);
+      Unused << SendUpdatePlatformDNSInformation(suffixList);
     }
 
     if (!strcmp(aTopic, NS_NETWORK_LINK_TOPIC) && mURISetByDetection) {
-      Unused << SendNotifyObserver(
-          nsDependentCString(aTopic),
-          aData ? nsDependentString(aData) : VoidString());
       CheckURIPrefs();
     }
   }
@@ -179,6 +134,7 @@ void TRRServiceParent::PrefsChanged(const char* aName, void* aSelf) {
 
 void TRRServiceParent::prefsChanged(const char* aName) {
   if (!aName || !strcmp(aName, "network.trr.uri") ||
+      !strcmp(aName, "network.trr.default_provider_uri") ||
       !strcmp(aName, kRolloutURIPref)) {
     OnTRRURIChange();
   }
